@@ -400,7 +400,6 @@ QList<QByteArray> QWindowsAudioDeviceInfo::availableDevices(QAudio::Mode mode)
     Q_UNUSED(mode)
 
     QList<QByteArray> devices;
-#ifndef Q_OS_WINCE
     //enumerate device fullnames through directshow api
     CoInitialize(NULL);
     ICreateDevEnum *pDevEnum = NULL;
@@ -454,55 +453,18 @@ QList<QByteArray> QWindowsAudioDeviceInfo::availableDevices(QAudio::Mode mode)
         pDevEnum->Release();
     }
     CoUninitialize();
-#else // Q_OS_WINCE
-    if (mode == QAudio::AudioOutput) {
-        WAVEOUTCAPS woc;
-        unsigned long iNumDevs,i;
-        iNumDevs = waveOutGetNumDevs();
-        for (i=0;i<iNumDevs;i++) {
-            if (waveOutGetDevCaps(i, &woc, sizeof(WAVEOUTCAPS))
-                == MMSYSERR_NOERROR) {
-                QByteArray  device;
-                QDataStream ds(&device, QIODevice::WriteOnly);
-                ds << quint32(i) << QString::fromWCharArray(woc.szPname);
-                devices.append(device);
-            }
-        }
-    } else {
-        WAVEINCAPS woc;
-        unsigned long iNumDevs,i;
-        iNumDevs = waveInGetNumDevs();
-        for (i=0;i<iNumDevs;i++) {
-            if (waveInGetDevCaps(i, &woc, sizeof(WAVEINCAPS))
-                == MMSYSERR_NOERROR) {
-                QByteArray  device;
-                QDataStream ds(&device, QIODevice::WriteOnly);
-                ds << quint32(i) << QString::fromWCharArray(woc.szPname);
-                devices.append(device);
-            }
-        }
-    }
-#endif // !Q_OS_WINCE
 
     return devices;
 }
 
-QByteArray QWindowsAudioDeviceInfo::defaultOutputDevice()
+QByteArray QWindowsAudioDeviceInfo::defaultDevice(QAudio::Mode mode)
 {
+    const QString &name = (mode == QAudio::AudioOutput) ? QStringLiteral("Default Output Device")
+                                                        : QStringLiteral("Default Input Device");
     QByteArray defaultDevice;
     QDataStream ds(&defaultDevice, QIODevice::WriteOnly);
     ds << quint32(WAVE_MAPPER) // device ID for default device
-       << QStringLiteral("Default Output Device");
-
-    return defaultDevice;
-}
-
-QByteArray QWindowsAudioDeviceInfo::defaultInputDevice()
-{
-    QByteArray defaultDevice;
-    QDataStream ds(&defaultDevice, QIODevice::WriteOnly);
-    ds << quint32(WAVE_MAPPER) // device ID for default device
-       << QStringLiteral("Default Input Device");
+       << name;
 
     return defaultDevice;
 }
